# This code is provided as supplemental material for the publication of
# "Estimating Transmission Potential of H5N1 Viruses among Humans in
# Egypt Using Phylogeny, Genetic Distance and Sampling Time Interval"
# by Wessam Mohamed, Kimihito Ito, Ryosuke Omori

range_query <-function(x,gen,tim,geo){
    N<-vector("character");
    labels<-labels(mat_gen)[[1]];
    for(y in labels){
	if(mat_gen[x,y]<= gen & mat_tim[x,y]<= tim & mat_geo[x,y]<= geo){
	    N=c(N,y);
	}
    }
    return(N);
}

# DBSCAN algorithm
clustering <- function(gen,tim,geo){
  labels<-labels(mat_gen)[[1]];
  num_seq<-length(labels);
  
  C<-0;
  X<- rep(-1,num_seq);
  
  for(p in 1:num_seq){
    if(X[p]!=-1){
      next; 
    }
    N=range_query(labels[p],gen,tim,geo);
    if(length(N)<2){
      X[p]<-0;
      next; 
    }
    C<-C+1;
    X[p]<-C;
    S<-N[N!=p];
    while(length(S)!=0){
      q<-which(labels==S[1]);
      S<-S[S!=labels[q]];
      if(X[q]==0){
        X[q]<-C;
      }
      if(X[q]!=-1){
        next;
      }
      X[q]<-C;
      n=range_query(labels[q],gen,tim,geo);
      if(length(n)>=2){
        S<-c(S,n);
      }
    }
  }
  return(X)
}

calc_num_transmission<-function(X){
  num_singleton<-length(X[X==0]);
  num_cluster<-length(unique(X))-1+num_singleton;
  num_seq<-length(X);
  num_transmission<-num_seq-num_cluster;
  return(num_transmission)
}

logl <-function(n,S,p){
  if(n==0){
    value = (S-n)*log(1-p);
  }else if(n==S){
    value = n*log(p);
  }else{
    value = n*log(p)+(S-n)*log(1-p);
  }
  return(value);
}

mat_gen <-as.matrix(read.csv("Hu60_matrix.csv",header=TRUE,row.names=1));
mat_tim <-as.matrix(read.csv("time_diff.csv",header=TRUE,row.names=1));
mat_geo <-as.matrix(read.csv("dist_city.csv",header=TRUE,row.names=1));

#Threshold values
th_gen<-0.004488;
th_tim<-30;
th_geo<-0;

X<-clustering(th_gen,th_tim,th_geo);
num_transmission<-calc_num_transmission(X);
num_seq<-length(X);
R0 <- num_transmission/num_seq;

f <-function(p) -2*(logl(num_transmission,num_seq,p)-logl(num_transmission,num_seq,R0))-3.84;

if(R0==0){
    lb=0;
 }else{
    lb=uniroot(f,c(0,R0))$root;
 }
if(R0==1){
    ub=1;
 }else{
    ub=uniroot(f,c(R0,1))$root;
 }

print(R0);
print(lb);
print(ub);


